;
; Creation par Alain Coulais (AC) le 17/12/2003 a Meudon
; Large refonte par AC en Avril 2005
;
; -----------------------------------------------------------
;
; Large refonte derniere semaine Avril 2005 (AC)
;
; - Correction de l'equation Pfuite suite a la visite de A. Woodcraft
; - Gestion supposee correcte de T_bath (Temperature effective de
;   la platine dite 100 mK), desormais distincte de T_100mk
;   (pb du a g_s0 car g_s0 est definit d'apres T_100mk)
; - Suppression de larges parts de codes desormais superflues grace a
;   la struture "Bolo"
; - Gestion correcte du courant (le champ "courant"
;   de la structure "Bolo" n'est pas pertinent)
; - La gestion des tableaux "tab_i" et "tab_T_bath" est desormais
;   externalisee dans INIT_ARRAY
;
; Historique des modifications:
;
; le 11/05/2005: AC
; - gestion fine des valeurs du parametre Debug
;   si Debug=1, sauvegarde des variables intermediares
;   si Debug=2, modification du courant
;
; le 02/05/2005: AC
; - suppresion des anciens commentaires (si necessaire,
;   cf Equa_Temp_Bolo.pro et bolo_direct_modelOLD.pro)
;
; -----------------------------------------------------------
;
; Dependances (Ce programme necessite):
; AClib: AC_SETMESSAGES, INIT_ARRAY (OUT_OF_RANGE)
; -----------------------------------------------------------
;
function BOLO_DIRECT_MODEL, OptSignal, bolo=bolo, T_bath=T_bath, $
                            delta_t=delta_t, current=current, $
                            display=display, verbose=verbose, $
                            debug=debug, test=test, help=help
;
mess=AC_SETMESSAGES('BOLO_DIRECT_MODEL')
;
if (N_PARAMS() lt 1) then begin
	print, mess.fatal+'missing arguments'
	help=1
endif
;
if KEYWORD_SET(help) then begin
    print, 'function BOLO_DIRECT_MODEL, OptSignal, delta_t=delta_t, $'
    print, '                 current=current, T_bath=T_bath, $'
    print, '                 bolo=bolo, $'
    print, '                 display=display, verbose=verbose, $'    
    print, '                 debug=debug, test=test, help=help'
    print, ' '
    print, 'Program Input are :'
    print, '- OptSignal : [No Default] optical signal (Watts) (1D array)'
    print, '- delta_t   : [Default 1e-4] sampling time (second) (Number)'
    print, '- current   : [Default 1e-9 A = 1 nA] current (Ampere) (Number or 1D array)'
    print, '- T_bath    : [Default 100 mK] temperature of the so-called 100 mK platin (Kelvin) (Number or 1D array)'
    print, '- bolo      : [No Default] bolo structure (Read with bolo_list=LIST_BOLO_READFILE())'
    print, ''
    print, 'Program Output is:'
    print, '- temperature of the bolometer (Kelvin)  (1D array)'
    return, -1
endif
;
; -----------------------------------
; gestion du mot-clef Debug
;   si Debug=1, sauvegarde des variables intermediares
;     (tableaux Pj, C, Pf)
;   si Debug=2, modification du courant
;
if (N_ELEMENTS(debug) NE 1) then debug=0
;
; -----------------------------------
; Because of numerical problems, it is, up to know,
; mandatory to compute in Double Precision
;
double_precision=1
;
; -----------------------------------
; Gestion du Signal Optique
; ToDo : 1/ il faut que ca soit un tableau 1D positif
;
nbp=N_ELEMENTS(OptSignal)
;
; -----------------------------------
;
; Gestion du pas en temps (time sampling)
; ToDo : 1/ doit pouvoir evoluer vers un system 1D (array) regulier
; ToDo : 2/ doit pouvoir evoluer vers un system 1D (array) IRregulier
;
if (N_ELEMENTS(delta_t) NE 1) then delta_t=1e-4
if (delta_t LE 0.) then begin
    print, mess.fatal+'"delta_t" must be positive'
    return, -1
endif
;
; -------------------------------------------------
; AC, modif des noms des variable le 03/03/04, d'apres XML IMO
;
if (N_ELEMENTS(bolo) EQ 0) then begin
    print, mess.fatal+"missing mandatory parameter <<bolo>>"
    print, mess.info+"A list of bolo can be read with bolo_list=LIST_BOLO_READFILE()"
    return, -1
endif
if (N_ELEMENTS(bolo) GT 1) then begin
    print, mess.fatal+"One and only one bolo should be provided"
    print, mess.info+"One bolo in a bolo list can be extracted using "
    print, mess.info+"bolo_solo=LIST_BOLO_EXTRACT(bolo_list, name='', freq='') ..."
    return, -1
endif
;
if (N_ELEMENTS(bolo) EQ 1) then begin
    r0=bolo.Rinf
    Tg=bolo.Ttrans
    m=bolo.Alpha
    g_s0=bolo.Gbol
    beta=bolo.Beta
    c0=bolo.c0
    dcapa=bolo.dcapa
endif
;
if KEYWORD_SET(double_precision) then begin
    r0=double(r0)
    Tg=Double(Tg)
    g_s0=Double(g_s0)
    c0=Double(c0)
endif
;
if KEYWORD_SET(verbose) then begin
    print, mess.info+'Bolo parameters values for Bolo', bolo.name, 'at Freq', bolo.freq
    print, mess.info+'P resistance (Tg, m, r0):', Tg, m, r0
    print, mess.info+'P interne Bolo (c0, dcapa):', c0, dcapa
    print, mess.info+'P fuite (g_s0, beta):', g_s0, beta
endif
;
; ------------------------------------------------------
; We need T_100mk for processing Pfuite from g_s0 (M Piat conventions)
;
T_100mk=DOUBLE(0.1)
CoefPf=g_s0/((T_100mk^beta)*(beta+1.))
beta_et_1=beta+1.
;
; ------------------------------------------------------
;
; We need to know the value of the current
; If this information is not provide, we use a Default Value
; Current can be a single value (constant) OR a 1D-Array
; (if 1D-Array same size than Time and T_bath)
;
; si on ne fournit qu'une seule valeur, on le transforme en tableau
; il n'y a pas de probleme de signe puisque qu'on se sert de son carre
;
tab_i=INIT_ARRAY(current, nbp, default_val=1e-9, $
                     min_val=1e-12, max_val=1e-7, $
                     name_of_val='current')
;
if KEYWORD_SET(verbose) then print, mess.info+'mean Current is:', MEAN(tab_i)
;
; ------------------------------------------------------
;
; T_bath can be different that T_100mk
; T_bath can change with time 
; If nothing is provide for T_bath, we set it to T_100mk
;
tab_T_bath=INIT_ARRAY(T_bath, nbp, default_val=T_100mk, $
                      min_val=0.050, max_val=0.200, $
                      name_of_val='T_bath')
;
if KEYWORD_SET(verbose) then print, mess.info+'mean T_bath:', MEAN(tab_T_bath)
;
; -------------------------------------------
; preparation des calculs; initialisation de tableaux
; 
; desormais, on a un tableau de courant, on peut pre-calculer ca
;
r0i2=r0*tab_i*tab_i
;
; creation et initialisation du tableau des temperatures
; du bolometre (ce que l'on calculera)
;
TempCalcul=DBLARR(nbp)
TempCalcul(0)=tab_T_bath(0)
;
; ---------------------
; ToDo: a changer pour pouvoir gerer un Delta_t non constant !
; AC, test Double Precision le 18/02/2004
; t=Delta_t*findgen(nbp)
;
t=Delta_t*Dindgen(nbp)
;
; ---------------------------------------------------------------------
;
; le calcul lui-meme, simple x(n+1)=x(n)+delta_t*Function(x(n), params)
;
if (debug EQ 1) then begin
    Pj_tab=Dblarr(nbp)
    C_tab=Dblarr(nbp)
    Pf_tab=Dblarr(nbp)
endif
;
for i=1L, (nbp-1L) do begin
    Temp=TempCalcul(i-1)
    Pj=r0i2(i)*EXP((Tg/Temp)^m)
    C=c0*(Temp/T_100mk)^dcapa
    Pf=CoefPf*(Temp^beta_et_1-tab_T_bath(i)^beta_et_1)
    ;;
    ;; the best (tested) way in order to increase numerical stability
    TempCalcul(i)=Temp+(Delta_t/c)*((Pj-Pf)+OptSignal(i))
    ;;
    ;;	if KEYWORD_SET(verbose) then print, 'Pj, C, Pf, Ti', $
    ;;	 	 Pj, C, Pf, TempCalcul(i)
    if (debug EQ 1) then begin
        Pj_tab(i)=Pj
        C_tab(i)=C
        Pf_tab(i)=Pf
    endif
endfor
;
if (debug EQ 2) then begin
    print, mess.warning+'input "current" has been change to "tab_i"'
    current=tab_i
endif
;
; fin du calcul lui-meme
; ------------------------------------------------
;
if KEYWORD_SET(display) then begin
    !P.multi=[0,0,2]
    window, xsize=900, ysize=900
    plot, t, OptSignal, title='Incoming flux'
    plot, t, TempCalcul, title='Temperature variation'
    !p.multi=0
endif
;
if KEYWORD_SET(test) then STOP
;
return, TempCalcul
;
end
;
